@@ -13,6 +13,7 @@ from trytond.transaction import Transaction
 from trytond.pool import Pool
 from trytond import backend
 from trytond.config import config
+from trytond.tools import lstrip_wildcard
 from trytond.tools.multivalue import migrate_property
 from trytond.modules.company.model import (
     CompanyMultiValueMixin, CompanyValueMixin)
@@ -239,8 +240,15 @@ class Product(
                                 fields.Many2Many))):
                     order_method = TemplateFunction.order(attr)
                     setattr(cls, 'order_%s' % attr, order_method)
+                if isinstance(tfield, fields.One2Many):
+                    getattr(cls, attr).setter = '_set_template_function'
 
-    @fields.depends('template', '_parent_template.id')
+    @classmethod
+    def _set_template_function(cls, products, name, value):
+        # Prevent NotImplementedError for One2Many
+        pass
+
+    @fields.depends('template')
     def on_change_template(self):
         for name, field in self._fields.items():
             if isinstance(field, TemplateFunction):
@@ -253,9 +261,6 @@ class Product(
     def get_template(self, name):
         value = getattr(self.template, name)
         if isinstance(value, Model):
-            field = getattr(self.__class__, name)
-            if field._type == 'reference':
-                return str(value)
             return value.id
         elif (isinstance(value, (list, tuple))
                 and value and isinstance(value[0], Model)):
@@ -305,8 +310,11 @@ class Product(
             bool_op = 'AND'
         else:
             bool_op = 'OR'
+        code_value = clause[2]
+        if clause[1].endswith('like'):
+            code_value = lstrip_wildcard(clause[2])
         return [bool_op,
-            ('code',) + tuple(clause[1:]),
+            ('code', clause[1], code_value) + tuple(clause[3:]),
             ('template.name',) + tuple(clause[1:]),
             ]
 
